<?php

if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'wp_easycart_admin' ) ) :

final class wp_easycart_admin{
	
	protected static $_instance = null;
	
	private $wpdb;
	
	public $preloader;
	
	public $month_sales_total;
	public $month_name;
	public $month_percentage_change;
	public $month_percentage_goal;
	public $month_goal_total;
	
	public $daily_sales;
	public $weekly_sales;
	public $monthly_sales;
	public $yearly_sales;
	
	public $daily_items_sold;
	public $weekly_items_sold;
	public $monthly_items_sold;
	public $yearly_items_sold;
	
	public $daily_abandonment;
	public $weekly_abandonment;
	public $monthly_abandonment;
	public $yearly_abandonment;
	
	public $new_orders;
	public $new_unviewed_orders;
	public $pending_reviews;
	public $cart_users;
	
	public $settings;
	public $shipping_zones;
	public $shipping_zones_items;
	public $countries;
	public $states;
	
	public $store_page;
	public $cart_page;
	public $account_page;
	public $permalink_divider;
	
	public static function instance( ) {
		
		if( is_null( self::$_instance ) ) {
			self::$_instance = new self(  );
		}
		return self::$_instance;
	
	}
	
	public function __construct( ){ 
	
		if( !defined( 'WP_EASYCART_ADMIN_PLUGIN_DIR' ) )
			define( 'WP_EASYCART_ADMIN_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
			
		if( ! defined( 'WP_EASYCART_ADMIN_PLUGIN_URL' ) )
			define( 'WP_EASYCART_ADMIN_PLUGIN_URL', plugin_dir_url( __FILE__ ) );

		if( ! defined( 'WP_EASYCART_ADMIN_PLUGIN_FILE' ) )
			define( 'WP_EASYCART_ADMIN_PLUGIN_FILE', __FILE__ );
	
		if( !defined( 'WP_EASYCART_ADMIN_DB_VERSION' ) )
			define( 'WP_EASYCART_ADMIN_DB_VERSION', 0.1 );
			
		// Keep reference to wpdb
		global $wpdb;
		$this->wpdb =& $wpdb;
		
		$this->preloader = new wp_easycart_admin_preloader( );
		$this->helpsystem = new wp_easycart_admin_online_docs( );
		
		// Setup Basic Variables for Admin Design
		$this->month_sales_total = $this->get_month_sales_total( );
		$this->month_name = date( 'F' );
		$this->month_percentage_change = $this->get_month_percentage_change( );
		$this->month_goal_total =  get_option( 'ec_option_admin_sales_goal');
		$this->month_percentage_goal = ( $this->month_sales_total / $this->month_goal_total ) * 100;
		if( $this->month_percentage_goal > 100 ) 
			$this->month_percentage_goal = 100;
		
		// Sales Data
		$this->daily_sales = $this->get_daily_sales( );
		$this->weekly_sales = $this->get_weekly_sales( );
		$this->monthly_sales = $this->get_monthly_sales( );
		$this->yearly_sales = $this->get_yearly_sales( );
		
		// Cart Items Sold Data
		$this->daily_items_sold = $this->get_daily_items_sold( );
		$this->weekly_items_sold = $this->get_weekly_items_sold( );
		$this->monthly_items_sold = $this->get_monthly_items_sold( );
		$this->yearly_items_sold = $this->get_yearly_items_sold( );
		
		// Cart Abandonment Data
		$this->daily_abandonment = $this->get_daily_abandonment( );
		$this->weekly_abandonment = $this->get_weekly_abandonment( );
		$this->monthly_abandonment = $this->get_monthly_abandonment( );
		$this->yearly_abandonment = $this->get_yearly_abandonment( );
		
		// Basic Stats
		$this->new_unviewed_orders = $this->get_total_new_unviewed_orders();
		$this->new_orders = $this->get_total_new_orders( );
		$this->pending_reviews = $this->get_total_new_reviews( );
		$this->cart_users = $this->get_total_cart_users( );
		
		// EasyCart Admin Actions
		add_action( 'wp_easycart_admin_upsell_popup', array( $this, 'load_upsell_popup' ) );
		add_action( 'wp_easycart_admin_mobile_navigation', array( $this, 'load_mobile_navigation' ), 1, 0 );
		add_action( 'wp_easycart_admin_left_navigation', array( $this, 'load_left_navigation' ), 1, 0 );
		add_action( 'wp_easycart_admin_head_navigation', array( $this, 'load_head_navigation' ), 1, 0 );
		add_action( 'wp_easycart_admin_messages', array( $this, 'print_admin_message' ) );

		// Hook Actions
		add_action( 'admin_init', array( $this, 'delete_gateway_log' ) );
		add_action( 'admin_init', array( $this, 'complete_init' ) );
		add_action( 'admin_init', array( $this, 'setup_pro_hooks' ) );
		add_action( 'admin_init', array( $this, 'process_actions' ) );
		add_action( 'admin_menu', array( $this, 'setup_menu' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'load_scripts' ) );
		add_action( 'admin_notices', array( $this, 'wp_easycart_pro_check' ) );
		
		// WordPress Filters
		add_filter( 'admin_title', array( $this, 'set_title' ), 10, 2 );
	}
	
	public function delete_gateway_log( ){ 
		if( current_user_can( 'manage_options' ) && isset( $_GET['ec_admin_form_action'])  && $_GET['ec_admin_form_action'] == 'ec_delete_gateway_log' ){
			wp_easycart_admin_miscellaneous( )->delete_gateway_log( );
		}
	}
	
	public function complete_init( ){
		
		// Link Information
		$store_page_id = get_option('ec_option_storepage');
		$cart_page_id = get_option('ec_option_cartpage');
		$account_page_id = get_option('ec_option_accountpage');
		
		if( function_exists( 'icl_object_id' ) ){
			$store_page_id = icl_object_id( $store_page_id, 'page', true, ICL_LANGUAGE_CODE );
			$cart_page_id = icl_object_id( $cart_page_id, 'page', true, ICL_LANGUAGE_CODE );
			$account_page_id = icl_object_id( $account_page_id, 'page', true, ICL_LANGUAGE_CODE );
		}
		
		$this->store_page = get_permalink( $store_page_id );
		$this->cart_page = get_permalink( $cart_page_id );
		$this->account_page = get_permalink( $account_page_id );
		
		if( class_exists( "WordPressHTTPS" ) && isset( $_SERVER['HTTPS'] ) ){
			$https_class = new WordPressHTTPS( );
			$this->store_page = $https_class->makeUrlHttps( $this->store_page );
			$this->cart_page = $https_class->makeUrlHttps( $this->cart_page );
			$this->account_page = $https_class->makeUrlHttps( $this->account_page );
		}
		
		if( substr_count( $this->cart_page, '?' ) )					$this->permalink_divider = "&";
		else														$this->permalink_divider = "?";
		
	}
	
	public function process_actions( ){ 
		if( current_user_can( 'manage_options' ) && (isset( $_POST['ec_admin_form_action'] ) || isset( $_GET['ec_admin_form_action'] )) ){
			$actions = new wp_easycart_admin_actions( );
			$actions->process_action( );
		}
		
	}
	
	/* STATS FUNCTIONS */
	private function get_month_sales_total( ){
		return $this->wpdb->get_var( "SELECT SUM( ec_order.grand_total - ec_order.refund_total ) as total FROM ec_order WHERE MONTH( NOW( ) ) = MONTH( ec_order.order_date ) AND YEAR( NOW( ) ) = YEAR( ec_order.order_date )" );
	}
	
	private function get_month_percentage_change( ){
		$last_month = $this->wpdb->get_var( "SELECT SUM( ec_order.grand_total - ec_order.refund_total ) as total FROM ec_order WHERE MONTH( DATE_SUB( NOW( ), INTERVAL 30 DAY ) ) = MONTH( ec_order.order_date ) AND YEAR( DATE_SUB( NOW( ), INTERVAL 30 DAY ) ) = YEAR( ec_order.order_date )" );
		if($last_month == null) $last_month = 0;
		$datestring = 'first day of last month';
		$dt = date_create( $datestring );
		$month = intval( date_format( $dt, 'm' ) );
		$year = intval( date_format( $dt, 'Y' ) );
		$days_last = $this->days_in_month( $month, $year );
		$days_this_month = intval( date( 'j' ) );
		
		if( ( $last_month / $days_last ) * $days_this_month == 0 )
			return 0;
		else
			return ( ( $this->month_sales_total / ( ( $last_month / $days_last ) * $days_this_month ) ) - 1 ) * 100; //compare total over same number of days between months
	}
	
	private function days_in_month( $month, $year ){ 
		return $month == 2 ? ( $year % 4 ? 28 : ( $year % 100 ? 29 : ( $year % 400 ? 28 : 29 ) ) ) : ( ( $month - 1 ) % 7 % 2 ? 30 : 31 ); 
	}
	
	private function get_daily_sales( ){
		$sales = array( );
		for( $i = 0; $i < 14; $i++ ){
			$sale_total = $this->wpdb->get_row( "SELECT IFNULL( SUM( ec_order.grand_total - ec_order.refund_total ), 0 ) as total, DATE_FORMAT( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ), '%m/%d/%Y') as date FROM ec_order LEFT JOIN ec_orderstatus ON ec_order.orderstatus_id = ec_orderstatus.status_id WHERE DAY( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = DAY( ec_order.order_date ) AND MONTH( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = MONTH( ec_order.order_date ) AND YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = YEAR( ec_order.order_date ) AND ec_orderstatus.is_approved = 1" );
			$sales[] = array( 'total' => $sale_total->total, 'date' => $sale_total->date );
		}
		return $sales;
	}
	
	private function get_weekly_sales( ){
		$sales = array( );
		for( $i = 0; $i < 10; $i++ ){
			$sale_total = $this->wpdb->get_row( "SELECT IFNULL( SUM( ec_order.grand_total - ec_order.refund_total ), 0 ) as total, DATE_FORMAT( DATE_SUB( NOW( ), INTERVAL " . $i . " WEEK ), '%m/%d/%Y') as date FROM ec_order LEFT JOIN ec_orderstatus ON ec_order.orderstatus_id = ec_orderstatus.status_id WHERE ec_order.order_date BETWEEN DATE_SUB( NOW( ), INTERVAL " . ( $i + 1 ) . " WEEK ) AND DATE_SUB( NOW( ), INTERVAL " . $i . " WEEK ) AND ec_orderstatus.is_approved = 1" );
			$sales[] = array( 'total' => $sale_total->total, 'date' => $sale_total->date );
		}
		return $sales;
	}
	
	private function get_monthly_sales( ){
		$sales = array( $this->month_name => $this->month_sales_total );
		for( $i = 1; $i < 10; $i++ ){
			$sale_total = $this->wpdb->get_row( "SELECT IFNULL( SUM( ec_order.grand_total - ec_order.refund_total ), 0 ) as total, MONTHNAME( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) as month_name FROM ec_order LEFT JOIN ec_orderstatus ON ec_order.orderstatus_id = ec_orderstatus.status_id WHERE MONTH( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) = MONTH( ec_order.order_date ) AND YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) = YEAR( ec_order.order_date ) AND ec_orderstatus.is_approved = 1" );
			$sales[$sale_total->month_name] = $sale_total->total;
		}
		return $sales;
	}
	
	private function get_yearly_sales( ){
		$sales = array( );
		for( $i = 0; $i < 10; $i++ ){
			$sale_total = $this->wpdb->get_row( "SELECT IFNULL( SUM( ec_order.grand_total - ec_order.refund_total ), 0 ) as total, YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " YEAR ) ) as year FROM ec_order LEFT JOIN ec_orderstatus ON ec_order.orderstatus_id = ec_orderstatus.status_id WHERE YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " YEAR ) ) = YEAR( ec_order.order_date ) AND ec_orderstatus.is_approved = 1" );
			$sales[] = array( 'total' => $sale_total->total, 'date' => $sale_total->year );
		}
		return $sales;
	}
	
	private function get_daily_items_sold( ){
		$items_sold = array( );
		for( $i = 0; $i < 14; $i++ ){
			$items = $this->wpdb->get_row( "SELECT IFNULL( SUM( ec_orderdetail.quantity ), 0 ) as total, DATE_FORMAT( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ), '%m/%d/%Y') as date FROM ec_orderdetail LEFT JOIN ec_order ON ec_order.order_id = ec_orderdetail.order_id LEFT JOIN ec_orderstatus ON ec_order.orderstatus_id = ec_orderstatus.status_id WHERE DAY( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = DAY( ec_order.order_date ) AND MONTH( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = MONTH( ec_order.order_date ) AND YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = YEAR( ec_order.order_date ) AND ec_orderstatus.is_approved = 1" );
			$items_sold[] = array( 'total' => $items->total, 'date' => $items->date );
		}
		return $items_sold;
	}
	
	private function get_weekly_items_sold( ){
		$items_sold = array( );
		for( $i = 0; $i < 10; $i++ ){
			$items = $this->wpdb->get_row( "SELECT IFNULL( SUM( ec_orderdetail.quantity ), 0 ) as total, DATE_FORMAT( DATE_SUB( NOW( ), INTERVAL " . $i . " WEEK ), '%m/%d/%Y') as date FROM ec_orderdetail LEFT JOIN ec_order ON ec_order.order_id = ec_orderdetail.order_id LEFT JOIN ec_orderstatus ON ec_order.orderstatus_id = ec_orderstatus.status_id WHERE ec_order.order_date BETWEEN DATE_SUB( NOW( ), INTERVAL " . ( $i + 1 ) . " WEEK ) AND DATE_SUB( NOW( ), INTERVAL " . $i . " WEEK ) AND ec_orderstatus.is_approved = 1" );
			$items_sold[] = array( 'total' => $items->total, 'date' => $items->date );
		}
		return $items_sold;
	}
	
	private function get_monthly_items_sold( ){
		$items_sold = array( );
		for( $i = 0; $i < 10; $i++ ){
			$items = $this->wpdb->get_row( "SELECT IFNULL( SUM( ec_orderdetail.quantity ), 0 ) as total, MONTHNAME( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) as month_name FROM ec_orderdetail LEFT JOIN ec_order ON ec_order.order_id = ec_orderdetail.order_id LEFT JOIN ec_orderstatus ON ec_order.orderstatus_id = ec_orderstatus.status_id WHERE MONTH( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) = MONTH( ec_order.order_date ) AND YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) = YEAR( ec_order.order_date ) AND ec_orderstatus.is_approved = 1" );
			$items_sold[] = array( 'total' => $items->total, 'date' => $items->month_name );
		}
		return $items_sold;
	}
	
	private function get_yearly_items_sold( ){
		$items_sold = array( );
		for( $i = 0; $i < 10; $i++ ){
			$items = $this->wpdb->get_row( "SELECT IFNULL( SUM( ec_orderdetail.quantity ), 0 ) as total, YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " YEAR ) ) as year FROM ec_orderdetail LEFT JOIN ec_order ON ec_order.order_id = ec_orderdetail.order_id LEFT JOIN ec_orderstatus ON ec_order.orderstatus_id = ec_orderstatus.status_id WHERE YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " YEAR ) ) = YEAR( ec_order.order_date ) AND ec_orderstatus.is_approved = 1" );
			$items_sold[] = array( 'total' => $items->total, 'date' => $items->year );
		}
		return $items_sold;
	}
	
	private function get_daily_abandonment( ){
		$abandonments = array( );
		for( $i = 0; $i < 14; $i++ ){
			$abandoned_total = $this->wpdb->get_row( "SELECT COUNT( ec_tempcart.session_id ) as total, DATE_FORMAT( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ), '%m/%d/%Y') as date FROM ec_tempcart WHERE DAY( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = DAY( ec_tempcart.last_changed_date ) AND MONTH( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = MONTH( ec_tempcart.last_changed_date ) AND YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " DAY ) ) = YEAR( ec_tempcart.last_changed_date )" );
			$abandonments[] = array( 'total' => $abandoned_total->total, 'date' => $abandoned_total->date );
		}
		return $abandonments;
	}
	
	private function get_weekly_abandonment( ){
		$abandonments = array( );
		for( $i = 0; $i < 10; $i++ ){
			$abandoned_total = $this->wpdb->get_row( "SELECT COUNT( ec_tempcart.session_id ) as total, DATE_FORMAT( DATE_SUB( NOW( ), INTERVAL " . $i . " WEEK ), '%m/%d/%Y') as date FROM ec_tempcart WHERE WEEK( DATE_SUB( NOW( ), INTERVAL " . $i . " WEEK ) ) = WEEK( ec_tempcart.last_changed_date ) AND YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " WEEK ) ) = YEAR( ec_tempcart.last_changed_date )" );
			$abandonments[] = array( 'total' => $abandoned_total->total, 'date' => $abandoned_total->date );
		}
		return $abandonments;
	}
	
	private function get_monthly_abandonment( ){
		$abandonments = array( );
		for( $i = 0; $i < 10; $i++ ){
			$abandoned_total = $this->wpdb->get_row( "SELECT COUNT( ec_tempcart.session_id ) as total, MONTHNAME( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) as date FROM ec_tempcart WHERE MONTH( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) = MONTH( ec_tempcart.last_changed_date ) AND YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " MONTH ) ) = YEAR( ec_tempcart.last_changed_date )" );
			$abandonments[] = array( 'total' => $abandoned_total->total, 'date' => $abandoned_total->date );
		}
		return $abandonments;
	}
	
	private function get_yearly_abandonment( ){
		$abandonments = array( );
		for( $i = 0; $i < 10; $i++ ){
			$abandoned_total = $this->wpdb->get_row( "SELECT COUNT( ec_tempcart.session_id ) as total, YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " YEAR ) ) as date FROM ec_tempcart WHERE YEAR( DATE_SUB( NOW( ), INTERVAL " . $i . " YEAR ) ) = YEAR( ec_tempcart.last_changed_date )" );
			$abandonments[] = array( 'total' => $abandoned_total->total, 'date' => $abandoned_total->date );
		}
		return $abandonments;
	}
	
	private function get_total_new_unviewed_orders( ){
		return $this->wpdb->get_var( "SELECT COUNT( ec_order.order_id ) as total FROM ec_order WHERE ec_order.order_viewed = 0" );
	}
	private function get_total_new_orders( ){
		return $this->wpdb->get_var( "SELECT COUNT( ec_order.order_id ) as total FROM ec_order WHERE ec_order.order_date > DATE_SUB( NOW( ), INTERVAL 1 WEEK )" );
	}
	
	private function get_total_new_reviews( ){
		return $this->wpdb->get_var( "SELECT COUNT( ec_review.review_id ) as total FROM ec_review WHERE ec_review.approved = 0" );
	}
	
	private function get_total_cart_users( ){
		return $this->wpdb->get_var( "SELECT COUNT( ec_user.user_id ) as total FROM ec_user" );
	}
	/* END STATS FUNCTIONS */
	
	public function setup_menu( ){
		
		if( function_exists( 'wp_easycart_admin_license' ) ){
			$license = wp_easycart_admin_license( )->license_check();
		}
		
		if( !function_exists( 'wp_easycart_admin_license' ) || ( function_exists( 'wp_easycart_admin_license' ) && !wp_easycart_admin_license( )->active_license ) ){
			$registration_count = 1;
			$registration_label = sprintf( __( 'Registration %s' ), "<span class='update-plugins count-$registration_count' title='License'><span class='update-count'>" . number_format_i18n($registration_count) . "</span></span>" );
		} else {
			$registration_count = 0;
			$registration_label = sprintf( __( 'Registration %s' ), "<span class='update-plugins count-$registration_count' title='License'><span class='update-count'>" . number_format_i18n($registration_count) . "</span></span>" );
		}
		
		//new unread order notification
		$orders_count = $this->new_unviewed_orders;
		$order_label = sprintf( __( 'Orders %s' ), "<span class='update-plugins count-$orders_count' title='New Orders'><span class='update-count'>" . number_format_i18n($orders_count) . "</span></span>" );
		
		//total notifications
		$total_notifications = $registration_count + $orders_count;
		$mainmenu_label = sprintf( __( 'WP EasyCart %s' ), "<span class='update-plugins count-$total_notifications' title='New Orders'><span class='update-count'>" . number_format_i18n($total_notifications) . "</span></span>" );


		add_menu_page( 'WP EasyCart', $mainmenu_label, 'manage_options', 'wp-easycart-dashboard', array( $this, 'load_dashboard' ), 'dashicons-cart', 58 );
		add_menu_page( 'Extensions', 'Extensions', 'manage_options', 'ec_adminv2', array( $this, 'load_extensions_page' ), 'dashicons-cart', 59 );
	
		add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Dashboard', 'Dashboard', 'manage_options', 'wp-easycart-dashboard', array( $this, 'load_dashboard' ) );
		//add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Reports', 'Reports', 'manage_options', 'wp-easycart-reports', array( $this, 'load_reports' ) );
		add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Products', 'Products', 'manage_options', 'wp-easycart-products', array( $this, 'load_products' ) );
		add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Orders', $order_label, 'manage_options', 'wp-easycart-orders', array( $this, 'load_orders' ) );
		add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Users', 'Users', 'manage_options', 'wp-easycart-users', array( $this, 'load_users' ) );
		add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Marketing', 'Marketing', 'manage_options', 'wp-easycart-rates', array( $this, 'load_marketing' ) );
		add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Settings', 'Settings', 'manage_options', 'wp-easycart-settings', array( $this, 'load_settings' ) );
		//add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Extensions', 'Extensions', 'manage_options', 'wp-easycart-extensions', array( $this, 'load_extensions' ) );
		add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Store Status', 'Store Status', 'manage_options', 'wp-easycart-status', array( $this, 'load_status' ) );
		add_submenu_page( 'wp-easycart-dashboard', 'WP EasyCart Registration', $registration_label , 'manage_options', 'wp-easycart-registration', array( $this, 'load_registration' ) );
		  
	}
	
	public function setup_pro_hooks( ){
		/* Products Tab*/
		add_action( 'wp_easycart_admin_subscription_plans_list', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_subscription_plans_details', array( $this, 'show_upgrade' ) );
		
		/* Marketing Tab */
		add_action( 'wp_easycart_admin_subscriptions_list', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_subscriptions_details', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_downloads_list', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_downloads_details', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_abandon_cart_load', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_coupon_list', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_coupon_details', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_giftcard_list', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_giftcard_details', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_promotion_list', array( $this, 'show_upgrade' ) );
		add_action( 'wp_easycart_admin_promotion_details', array( $this, 'show_upgrade' ) );
		do_action( 'wp_easycart_admin_pro_ready' );
	}
	
	public function load_dashboard( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_dashboard_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_dashboard_content( ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/dashboard.php' );
	}
	
	public function load_extensions_page( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_extensions_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_reports( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_reports_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_reports_content( ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/reports.php' );
	}
	
	public function load_admin( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_admin_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_admin_content( ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/admin.php' );
	}
	
	public function load_settings( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_settings_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_products( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_products_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_orders( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_orders_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_users( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_users_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_marketing( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_marketing_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_status( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_status_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	public function load_registration( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_registration_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function init_shipping_data( ){
	
		$this->settings = $this->wpdb->get_row( "SELECT * FROM ec_setting" );
		$this->shipping_zones = $this->wpdb->get_results( "SELECT * FROM ec_zone ORDER BY zone_name ASC" );
		$this->shipping_zones_items = $this->wpdb->get_results( "SELECT ec_zone_to_location.zone_to_location_id, ec_zone_to_location.zone_id, ec_zone_to_location.iso2_cnt, ec_zone_to_location.code_sta, ec_country.name_cnt AS country_name, ec_state.name_sta AS state_name FROM ec_zone, ec_zone_to_location LEFT JOIN ec_country ON ec_country.iso2_cnt = ec_zone_to_location.iso2_cnt LEFT JOIN ec_state ON ( ec_state.code_sta = ec_zone_to_location.code_sta AND ec_state.idcnt_sta =  ec_country.id_cnt ) WHERE ec_zone.zone_id = ec_zone_to_location.zone_id ORDER BY ec_zone.zone_name ASC" );
		$this->countries = $this->wpdb->get_results( "SELECT * FROM ec_country ORDER BY sort_order ASC" );
		$this->states = $this->wpdb->get_results( "SELECT ec_state.*, ec_country.name_cnt as country_name FROM ec_state LEFT JOIN ec_country ON ec_country.id_cnt = ec_state.idcnt_sta ORDER BY ec_state.sort_order ASC" );
								
	}
		
	public function load_settings_content( ){
			
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/setup-actions.php' );
		$ec_admin_settings = new ec_admin_settings( );
		
		// Try to Process Form Actions if Needed
		if( isset( $_POST['ec_admin_settings_action'] ) ){
			$ec_admin_settings->process_form_action( $_POST['ec_admin_settings_action'] );
		}
		
		$this->init_shipping_data( );
		global $wpdb;
		if( !get_option( 'ec_option_setup_wizard_done' ) && $result = $wpdb->get_row( "SELECT product_id FROM ec_product LIMIT 1" ) ){
		   update_option( 'ec_option_setup_wizard_done', 1 );
    	}
		
		// Display Page Setup
		if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "setup-wizard" ){
			wp_easycart_admin_setup_wizard( )->load_setup_wizard( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "initial-setup" ){
			if( !get_option( 'ec_option_setup_wizard_done' ) ){
				wp_easycart_admin_setup_wizard( )->load_setup_wizard( );
			}else{
				wp_easycart_admin_initial_setup( )->load_initial_setup( );
			}
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "products" ){
			wp_easycart_admin_products( )->load_products_setup( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "tax" ){
			 wp_easycart_admin_taxes( )->load_tax_setup( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "shipping-settings" ){
			$shipping = new wp_easycart_admin_shipping( );
			$shipping->load_shipping_setup( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "shipping-rates" ){
			$shipping = new wp_easycart_admin_shipping( );
			$shipping->load_shipping_rates( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "payment" ){
			wp_easycart_admin_payments( )->load_payments( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "checkout" ){
			$checkout = new wp_easycart_admin_checkout( );
			$checkout->load_checkout( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "account" ){
			$account = new wp_easycart_admin_account( );
			$account->load_account( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "miscellaneous"){
			wp_easycart_admin_miscellaneous( )->load_miscellaneous( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "language-editor" ){
			$language_editor = new wp_easycart_admin_language_editor( );
			$language_editor->load_language( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "design" ){
			$design = new wp_easycart_admin_design( );
			$design->load_design( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "third-party" ){
			wp_easycart_admin_third_party( )->load_third_party( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "email-setup" ){
			$email = new wp_easycart_admin_email_settings( );
			$email->load_email( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "cart-importer" ){
			$cart = new wp_easycart_admin_cart_importer( );
			$cart->load_cart_importer( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "country" ){
			wp_easycart_admin_country( )->load_country_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "states" ){
			wp_easycart_admin_states( )->load_states_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "perpage" ){
			wp_easycart_admin_perpage( )->load_perpage_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "pricepoint" ){
			wp_easycart_admin_pricepoint( )->load_pricepoint_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "logs" ){
			wp_easycart_admin_logging( )->load_log_list( );
		}else{
			if( !get_option( 'ec_option_setup_wizard_done' ) ){
				wp_easycart_admin_setup_wizard( )->load_setup_wizard( );
			}else{
				wp_easycart_admin_initial_setup( )->load_initial_setup( );
			}
		}
	}
	
	public function load_products_content( ){
		if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "option" ){
			wp_easycart_admin_option( )->load_option_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "inventory" ){
			wp_easycart_admin_inventory( )->load_inventory_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "optionitems" ){
			wp_easycart_admin_option( )->load_optionitem_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "category" ){
			wp_easycart_admin_category( )->load_category_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "category-products" ){
			wp_easycart_admin_category( )->load_category_product_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "category-products-manage" ){
			wp_easycart_admin_category( )->load_category_product_manage_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "menus" ){
			wp_easycart_admin_menus( )->load_menus_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "submenus" ){
			wp_easycart_admin_menus( )->load_submenus_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "subsubmenus" ){
			wp_easycart_admin_menus( )->load_subsubmenus_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "manufacturers" ){
			wp_easycart_admin_manufacturers( )->load_manufacturers_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "reviews" ){
			wp_easycart_admin_reviews( )->load_reviews_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "subscriptionplans" ){
			wp_easycart_admin_subscription_plans( )->load_subscription_plans_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "products" ){
			wp_easycart_admin_products( )->load_products_list( );
		}else{
			wp_easycart_admin_products( )->load_products_list( );
		}
	}
	
	public function load_orders_content( ){
		if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "orders" ){
			wp_easycart_admin_orders( )->load_orders_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "subscriptions" ){
			$subscriptions = new wp_easycart_admin_subscriptions( );
			$subscriptions->load_subscriptions_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "downloads" ){
			$downloads = new wp_easycart_admin_downloads( );
			$downloads->load_downloads_list( );
		}else{
			wp_easycart_admin_orders( )->load_orders_list( );
		}
	}
	
	public function load_users_content( ){
		if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "accounts" ){
			wp_easycart_admin_users( )->load_users_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "user-roles" ){
			wp_easycart_admin_user_role( )->load_user_role_list( );
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "subscribers" ){
			wp_easycart_admin_subscribers( )->load_subscriber_list( );
		}else{
			wp_easycart_admin_users( )->load_users_list( );
		}
	}
	
	public function load_marketing_content( ){
		if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "gift-cards" ){
			$giftcards = new wp_easycart_admin_giftcards( );
			$giftcards = $giftcards->load_giftcards_list();
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "coupons" ){
			$coupons = new wp_easycart_admin_coupons( );
			$coupons = $coupons->load_coupons_list();
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "promotions" ){
			$promotions = new wp_easycart_admin_promotions( );
			$promotions = $promotions->load_promotions_list();
		}else if( isset( $_GET['subpage'] ) && $_GET['subpage'] == "abandon-cart" ){
			$abandon_cart = new wp_easycart_admin_abandon_cart( );
			$abandon_cart->load_abandon_cart( );
		}else{
			$giftcards = new wp_easycart_admin_giftcards( );
			$giftcards = $giftcards->load_giftcards_list();
		}
	}
	
	public function load_status_content( ){
		$store_status = new wp_easycart_admin_store_status( );
		$store_status->load_status( );
	}
	
	public function load_registration_content( ){
		$registration = new wp_easycart_admin_registration( );
		$registration->load_registration_status( );
	}
	
	public function load_shipping_form( $shipping_type ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/shipping/' . $shipping_type . '.php' );
	}
	
	public function load_extensions( ){
		add_action( 'wp_easycart_admin_shell_content', array( $this, 'load_extensions_content' ), 1, 0 );
		$this->load_admin_shell( );
	}
	
	public function load_extensions_content( ){
		wp_easycart_admin_extensions( )->load_extensions( );
	}
	
	private function load_admin_shell( ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/shell.php' );
	}
	
	public function load_mobile_navigation( ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/mobile_nav.php' );
	}
	
	public function load_left_navigation( ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/left_nav.php' );
	}
	
	public function load_head_navigation( ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/head_nav.php' );
	}
	
	public function set_title( $admin_title, $title ){
		if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "products" ) ){
			return 'WP EasyCart Products';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "inventory" ){
			return 'WP EasyCart Inventory';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && ( $_GET['subpage'] == "option" || $_GET['subpage'] == "optionitems" ) ){
			return 'WP EasyCart Option Sets';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && ( $_GET['subpage'] == "category" || $_GET['subpage'] == "category-products" || $_GET['subpage'] == "category-products-manage" ) ){
			return 'WP EasyCart Categories';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && ( $_GET['subpage'] == "menus" || $_GET['subpage'] == "submenus" || $_GET['subpage'] == "subsubmenus" ) ){
			return 'WP EasyCart Menus';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "manufacturers" ){
			return 'WP EasyCart Manufacturers';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "reviews" ){
			return 'WP EasyCart Product Reviews';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "subscriptionplans" ){
			return 'WP EasyCart Subscription Plans';
		
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-orders" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "orders" ) ){
			return 'WP EasyCart Orders';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-orders" && isset( $_GET['subpage'] )&& $_GET['subpage'] == "subscriptions" ){
			return 'WP EasyCart Subscriptions';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-orders" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "downloads" ){
			return 'WP EasyCart Downloads';
		
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-users" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "accounts" ) ){
			return 'WP EasyCart User Accounts';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-users" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "user-roles" ){
			return 'WP EasyCart User Roles';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-users" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "subscribers" ){
			return 'WP EasyCart Subscribers';
			
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-rates" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "gift-cards" ) ){
			return 'WP EasyCart Gift Cards';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-rates" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "coupons" ){
			return 'WP EasyCart Coupons';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-rates" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "promotions" ){
			return 'WP EasyCart Promotions';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-rates" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "abandon-cart" ){
			return 'WP EasyCart Abandoned Cart';
		
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "initial-setup" ) ){
			return 'WP EasyCart Initial Setup';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "products" ){
			return 'WP EasyCart Product Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "tax" ){
			return 'WP EasyCart Tax Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "shipping-settings" ){
			return 'WP EasyCart Shipping Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "shipping-rates" ){
			return 'WP EasyCart Shipping Rates';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "payment" ){
			return 'WP EasyCart Payment Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "checkout" ){
			return 'WP EasyCart Checkout Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "account" ){
			return 'WP EasyCart Account Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "miscellaneous" ){
			return 'WP EasyCart Additional Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "language-editor" ){
			return 'WP EasyCart Language Editor';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "design" ){
			return 'WP EasyCart Design Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "email-setup" ){
			return 'WP EasyCart Email Setup';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "third-party" ){
			return 'WP EasyCart Third Party Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "cart-importer" ){
			return 'WP EasyCart Cart Importer';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "country" ){
			return 'WP EasyCart Country Management';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "states" ){
			return 'WP EasyCart State Management';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "perpage" ){
			return 'WP EasyCart Per Page Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "pricepoint" ){
			return 'WP EasyCart Price Point Settings';
		}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "logs" ){
			return 'WP EasyCart Logs';
		
		}else{
			return $admin_title;
		}
	}
	
	public function load_scripts( ){
		
		if( current_user_can( 'manage_options' ) && isset( $_GET['page'] ) && ( substr( $_GET['page'], 0, 11 ) == "wp-easycart" || $_GET['page'] == 'ec_adminv2' ) ){
			
			$https_link = "";
			if( class_exists( "WordPressHTTPS" ) ){
				$https_class = new WordPressHTTPS( );
				$https_link = $https_class->getHttpsUrl() . '/wp-admin/admin-ajax.php';
			}else{
				$https_link = str_replace( "http://", "https://", admin_url( 'admin-ajax.php' ) );
			}
			
			wp_enqueue_media( );
			
			wp_enqueue_style( 'wp_easycart_select2_css', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/css/select2.min.css' );
			wp_enqueue_script( 'wp_easycart_select2_js', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.3/js/select2.min.js', array('jquery') );
 			
			wp_register_script( 'wp_easycart_admin_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/admin.js' ), array( 'jquery', 'wp-color-picker' ), EC_CURRENT_VERSION );
			wp_enqueue_script( 'wp_easycart_admin_js' );
			if( is_ssl( ) ){
				wp_localize_script( 'wp_easycart_admin_js', 'ajax_object', array( 'ajax_url' => $https_link ) );
			}else{
				wp_localize_script( 'wp_easycart_admin_js', 'ajax_object', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
			}
			
			wp_register_script( 'wp_easycart_charts_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/Chart.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
			wp_enqueue_script( 'wp_easycart_charts_js' );
			
			wp_register_script( 'wp_easycart_validation_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/validation.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
			wp_enqueue_script( 'wp_easycart_validation_js' );
			
			wp_enqueue_script( 'jquery-ui-datepicker' );
			wp_register_style( 'wpeasycart-jquery-ui-css', 'https://code.jquery.com/ui/1.11.2/themes/smoothness/jquery-ui.css' );
			wp_enqueue_style( 'wpeasycart-jquery-ui-css' );
			wp_enqueue_style( 'wp-color-picker' );
			
			if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "category" ){
				wp_register_script( 'wp_easycart_admin_category_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/category.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_category_js' );	
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-orders" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "orders" ) ){
				wp_register_script( 'wp_easycart_admin_orders_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/orders.js' ), array( 'jquery', 'jquery-ui-datepicker' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_orders_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "menus" ){
				wp_register_script( 'wp_easycart_admin_menus_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/menus.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_menus_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && isset( $_GET['subpage'] ) && ( $_GET['subpage'] == "option" || $_GET['subpage'] == "optionitems" ) ){
				wp_register_script( 'wp_easycart_admin_option_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/option.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_option_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-products" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "products" ) ){
				wp_register_script( 'wp_easycart_admin_product_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/products.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_product_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-users" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "accounts" || $_GET['subpage'] == "user-roles" ) ){
				wp_register_script( 'wp_easycart_admin_users_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/users.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_users_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && ( !isset( $_GET['subpage'] ) || $_GET['subpage'] == "initial-setup" ) ){
				wp_register_script( 'wp_easycart_admin_initial_setup_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/initial-setup.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_initial_setup_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "products" ){
				wp_register_script( 'wp_easycart_admin_products_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/products.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_products_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "tax" ){
				wp_register_script( 'wp_easycart_admin_tax_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/tax.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_tax_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "shipping-settings" ){
				wp_register_script( 'wp_easycart_admin_shipping_settings_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/shipping-settings.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_shipping_settings_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "shipping-rates" ){
				wp_register_script( 'wp_easycart_admin_shipping_rates_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/shipping-rates.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_shipping_rates_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "payment" ){
				wp_register_script( 'wp_easycart_admin_payment_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/payment.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_payment_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "checkout" ){
				wp_register_script( 'wp_easycart_admin_checkout_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/checkout.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_checkout_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "account" ){
				wp_register_script( 'wp_easycart_admin_account_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/account.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_account_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "miscellaneous" ){
				wp_register_script( 'wp_easycart_admin_miscellaneous_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/miscellaneous.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_miscellaneous_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "language-editor" ){
				wp_register_script( 'wp_easycart_admin_language_editor_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/language-editor.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_language_editor_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "design" ){
				wp_register_script( 'wp_easycart_admin_design_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/design.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_design_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "third-party" ){
				wp_register_script( 'wp_easycart_admin_third_party_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/third-party.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_third_party_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "email-setup" ){
				wp_register_script( 'wp_easycart_admin_email_settings_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/email-settings.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_email_settings_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "cart-importer" ){
				wp_register_script( 'wp_easycart_admin_cart_importer_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/cart-importer.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_cart_importer_js' );
			}else if( isset( $_GET['page'] ) && $_GET['page'] == "wp-easycart-settings" && isset( $_GET['subpage'] ) && $_GET['subpage'] == "pricepoint" ){
				wp_register_script( 'wp_easycart_admin_pricepoint_js', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/js/pricepoint.js' ), array( 'jquery' ), EC_CURRENT_VERSION );
				wp_enqueue_script( 'wp_easycart_admin_pricepoint_js' );
			}
			
			wp_register_style( 'wp_easycart_admin_css', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/css/admin.css' ), array( ), EC_CURRENT_VERSION );
			wp_enqueue_style( 'wp_easycart_admin_css' );
			
			wp_register_style( 'wp_easycart_upgrade_css', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/css/upgrade.css' ), array( ), EC_CURRENT_VERSION );
			wp_enqueue_style( 'wp_easycart_upgrade_css' );
			
			add_editor_style( );
			add_thickbox( );
			wp_enqueue_script('common');
			wp_enqueue_script( 'post' );
			wp_enqueue_script('jquery-color');
			wp_enqueue_script( 'editor' );
			wp_enqueue_script( 'media-upload' );
			wp_enqueue_script( 'tiny_mce' );
			wp_enqueue_script( 'editorremov' );
			wp_enqueue_script( 'editor-functions' );
				
		}
		
		wp_register_style( 'wp_easycart_editor_css', plugins_url( EC_PLUGIN_DIRECTORY . '/admin/css/editor.css' ), array( ), EC_CURRENT_VERSION );
		wp_enqueue_style( 'wp_easycart_editor_css' );
		
		wp_localize_script( 'wp_easycart_admin_js', 'wp_easycart_admin_vars', array(
			'ajaxURL' => admin_url( 'admin-ajax.php' ),
			'ec_option_newsletter_done' => ( ( get_option( 'ec_option_bcc_email_addresses' ) != 'youremail@url.com' || get_option( 'ec_option_newsletter_done' ) ) ? 1 : 0 ),
			'ec_option_currency'		=> get_option( 'ec_option_currency' )
		) );
	}
	
	public function wp_easycart_pro_check( ){
		$pro_plugin_base = 'wp-easycart-pro/wp-easycart-admin-pro.php';
		$pro_plugin_file = WP_PLUGIN_DIR . '/' . $pro_plugin_base;
		if( file_exists( $pro_plugin_file ) && !is_plugin_active( $pro_plugin_base ) ) {
			echo '<div class="updated">';
			echo '<p>WP EasyCart PRO is installed but NOT ACTIVATED. Please <a href="' . $this->get_pro_activation_link( ) . '">click here to activate your WP EasyCart PRO plugin</a>.</p>';
			echo '</div>';
		}
	}
	
	public function get_pro_activation_link( ){ 
		$pro_plugin_file = WP_PLUGIN_DIR . '/wp-easycart-pro/wp-easycart-admin-pro.php';
		if( strpos( $pro_plugin_file, '/' ) ){
			$pro_plugin_file = str_replace( '/', '%2F', $pro_plugin_file );
		}
		$activate_url = sprintf( admin_url( 'plugins.php?action=activate&plugin=%s&plugin_status=all&paged=1&s' ), $pro_plugin_file ); 
		$_REQUEST['plugin'] = $pro_plugin_file;
		$activate_url = wp_nonce_url( $activate_url, 'activate-plugin_' . $pro_plugin_file );
		return $activate_url;
	}
	
	public function print_admin_message( ){
		$success_messages = apply_filters( 'wp_easycart_admin_success_messages', array( ) );
		$warning_messages = apply_filters( 'wp_easycart_admin_warning_messages', array( ) );
		$error_messages = apply_filters( 'wp_easycart_admin_error_messages', array( ) );
		
		if( count( $success_messages ) > 0 ){
			echo '<div id="ec_message" class="ec_admin_message_success"><div class="dashicons-before dashicons-thumbs-up"></div>' . implode( ', ', $success_messages ) . '</div>';
		
		}else if( count( $warning_messages ) > 0 ){
			echo '<div id="ec_message" class="ec_admin_message_warning"><div class="dashicons-before dashicons-warning"></div>' . implode( ', ', $warning_messages ) . '</div>';
		
		}else if( count( $error_messages ) > 0 ){
			echo '<div id="ec_message" class="ec_admin_message_error"><div class="dashicons-before dashicons-thumbs-down"></div>' . implode( ', ', $error_messages ) . '</div>';
		
		}
		
		if( !get_option( 'ec_option_allow_tracking' ) ){
			echo '<div id="ec_message" class="ec_admin_message_success ec_admin_allow_tracking">Please help improve WP EasyCart by sending us <a href="https://www.wpeasycart.com/terms-and-conditions/" target="_blank">basic usage data</a> for your plugin. <a href="admin.php?page=wp-easycart-settings&subpage=miscellaneous&ec_admin_form_action=allow-usage-tracking" class="ec_admin_tracking_allow" onclick="wp_easycart_allow_tracking( ); jQuery( this ).parent( ).fadeOut( ); return false;">allow</a><a href="admin.php?page=wp-easycart-settings&subpage=miscellaneous&ec_admin_form_action=deny-usage-tracking" class="ec_admin_tracking_deny" onclick="wp_easycart_deny_tracking( ); jQuery( this ).parent( ).fadeOut( ); return false;">deny</a></div>';
		}
	}
	
	public function load_upsell_popup( ){
		echo '<div id="ec_admin_upsell_popup"><div class="ec_admin_upsell_popup_close"><a href="#" onclick="hide_pro_required( ); return false;"><div class="dashicons-before dashicons-dismiss"></div></a></div><div class="ec_admin_upsell_popup_inner"><div class="ec_admin_upsell_popup_content">';
		$this->show_upgrade( );
		echo '<div style="clear:both;"></div></div></div></div>';
		echo '<script>jQuery( document.getElementById( \'ec_admin_upsell_popup\' ) ).appendTo( document.body );</script>';
	}
	
	public function show_upgrade( ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/upgrade/upgrade-screen.php' );
	}
	
	public function redirect( $page, $subpage, $args ){
		$url = $this->get_redirect_url( $page, $subpage, $args );
		wp_redirect( $url );
		exit( );
	}
	
	private function get_redirect_url( $page, $subpage, $args ){
		$url = "admin.php?page=" . $page . "&subpage=" . $subpage;
		foreach( $args as $key => $value ){
			$url .= "&" . $key . '=' . $value;
		}
		return $url;
	}
	
}
endif; // End if class_exists check

function wp_easycart_admin( ){
	return wp_easycart_admin::instance( );
}
wp_easycart_admin( );

add_action( 'wp_ajax_ec_admin_ajax_popup_newsletter_close', 'ec_admin_ajax_popup_newsletter_close' );
function ec_admin_ajax_popup_newsletter_close( ){
	if( isset( $_POST['wpeasycart_newsletter_customeremail'] ) && isset( $_POST['wpeasycart_join_newsletter'] ) && $_POST['wpeasycart_join_newsletter'] == '1' ){
		$customeremail = $_POST['wpeasycart_newsletter_customeremail'];
		$customername = get_bloginfo( 'name' );
		$site_url = site_url( );
		$site_url = str_replace( 'http://', '', $site_url );
		$site_url = str_replace( 'https://', '', $site_url );
		$site_url = str_replace( 'www.', '', $site_url );
		file_get_contents( sprintf( 'https://support.wpeasycart.com/licensing/activatetrial.php?customeremail=%s&customername=%s&siteurl=%s', urlencode( esc_attr( $customeremail ) ), urlencode( esc_attr( $customername ) ), urlencode( esc_attr( $site_url ) ) ) );
	}
	if( isset( $_POST['wpeasycart_newsletter_customeremail'] ) ){
		update_option( 'ec_option_bcc_email_addresses', esc_attr( $_POST['wpeasycart_newsletter_customeremail'] ) );
	}
	update_option( 'ec_option_newsletter_done', 1 );
	die( );
}

add_action( 'wp_ajax_ec_admin_ajax_allow_tracking', 'ec_admin_ajax_allow_tracking' );
function ec_admin_ajax_allow_tracking( ){
	update_option( 'ec_option_allow_tracking', '1' );
	if( !function_exists( 'wp_easycart_admin_tracking' ) ){
		include( WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/inc/wp_easycart_admin_tracking.php' );
	}
	do_action( 'wpeasycart_admin_usage_tracking_accepted' );
	die( );
}

add_action( 'wp_ajax_ec_admin_ajax_deny_tracking', 'ec_admin_ajax_deny_tracking' );
function ec_admin_ajax_deny_tracking( ){
	update_option( 'ec_option_allow_tracking', '-1' );
	die( );
}
?>