<?php
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'wp_easycart_admin_payments' ) ) :

final class wp_easycart_admin_payments{
	
	protected static $_instance = null;
	
	private $wpdb;
	
	public $payments_file;
	public $manual_bill_file;
	public $third_party_file;
	public $live_gateway_file;
	public $accepted_cards_file;
	public $upgrade_file;
	public $payments_dir;
		
	public $third_party_gateways;
	public $live_gateways;
	
	public $cart_page;
	public $permalink_divider;
	
	public static function instance( ) {
		
		if( is_null( self::$_instance ) ) {
			self::$_instance = new self(  );
		}
		return self::$_instance;
	
	}
		
	public function __construct( ){
		// Keep reference to wpdb
		global $wpdb;
		$this->wpdb =& $wpdb;
		
		// Setup File Names 
		$this->success_messages_file 		= WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/payments/success-messages.php';
		$this->payments_file 				= WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/payments/payment.php';
		$this->manual_bill_file 			= WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/payments/manual-bill.php';
		$this->third_party_file 			= WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/payments/third-party.php';
		$this->live_gateway_file 			= WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/payments/live-gateway.php';
		$this->accepted_cards_file 			= WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/payments/accepted-cards.php';
		$this->upgrade_file 				= WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/upgrade/upgrade-simple.php';
		$this->payments_dir					= WP_PLUGIN_DIR . "/" . EC_PLUGIN_DIRECTORY . '/admin/template/settings/payments/';
		
		// Link Information
		$cart_page_id = get_option('ec_option_cartpage');
		if( function_exists( 'icl_object_id' ) ){
			$cart_page_id = icl_object_id( $cart_page_id, 'page', true, ICL_LANGUAGE_CODE );
		}
		$this->cart_page = get_permalink( $cart_page_id );
		if( class_exists( "WordPressHTTPS" ) && isset( $_SERVER['HTTPS'] ) ){
			$https_class = new WordPressHTTPS( );
			$this->cart_page = $https_class->makeUrlHttps( $this->cart_page );
		}
		if( substr_count( $this->cart_page, '?' ) )					$this->permalink_divider = "&";
		else														$this->permalink_divider = "?";
		
		// Setup Default Payment Options
		$this->third_party_gateways 		= array( "2checkout_thirdparty"			=> "2Checkout",
													 "dwolla_thirdparty"			=> "Dwolla",
													 "nets"							=> "Nets Nexaxept",
													 "payfort"						=> "Payfort",
													 "paymentexpress_thirdparty" 	=> "Payment Express PxPay 2.0",
													 "paypal"						=> "PayPal Standard",
													 "realex_thirdparty"			=> "Realex",
													 "redsys"						=> "Redsys",
													 "sagepay_paynow_za"			=> "SagePay Pay Now South Africa",
													 "skrill"						=> "Skrill",
													 "custom_thirdparty"			=> "Custom Gateway"
										    );
		$this->live_gateways 				= array( "authorize"					=> "Authorize.net",
													 "beanstream"					=> "Beanstream",
													 "braintree"					=> "Braintree S2S",
													 "chronopay"					=> "Chronopay",
													 "virtualmerchant"				=> "Converge (Virtual Merchant)",
													 "eway"							=> "Eway",
													 "firstdata"					=> "First Date Payeezy (e4)",
													 "goemerchant"					=> "GoeMerchant",
													 "intuit"						=> "Intuit Payments",
													 "migs"							=> "MIGS", 
													 "moneris_ca"					=> "Moneris Canada",
													 "moneris_us"					=> "Moneris USA",
													 "nmi"							=> "Network Merchants (NMI)",
													 "payline"						=> "Payline",
													 "paymentexpress"				=> "Payment Express PxPost",
													 "paypal_pro"					=> "PayPal PayFlow Pro",
													 "paypal_payments_pro"			=> "PayPal Payments Pro",
													 "paypoint"						=> "PayPoint", 
													 "realex"						=> "Realex",
													 "sagepay"						=> "Sagepay",
													 "sagepayus"					=> "Sagepay US",
													 "securenet"					=> "SecureNet",
													 "securepay"					=> "SecurePay",
													 "stripe"						=> "Stripe",
													 "square"						=> "Square",
													 "custom"						=> "Custom Payment Gateway" 
											);
		
		// Actions
		add_action( 'wpeasycart_admin_payments_success', array( $this, 'load_success_messages' ) );
		add_action( 'wpeasycart_admin_payment_settings', array( $this, 'load_manual_bill' ) );
		add_action( 'wpeasycart_admin_payment_settings', array( $this, 'load_third_party' ) );
		add_action( 'wpeasycart_admin_load_third_party_select_options', array( $this, 'load_third_party_combo' ) );
		add_action( 'wpeasycart_admin_load_third_party_settings', array( $this, 'load_third_party_settings' ) );
		add_action( 'wpeasycart_admin_payment_settings', array( $this, 'load_live_gateway' ) );
		add_action( 'wpeasycart_admin_load_live_gateway_select_options', array( $this, 'load_live_gateway_combo' ) );
		add_action( 'wpeasycart_admin_load_live_gateway_settings', array( $this, 'load_live_gateway_settings' ) );
		add_action( 'wpeasycart_admin_payment_settings', array( $this, 'load_accepted_cards' ) );
		add_action( 'init', array( $this, 'save_settings' ) );
	}
	
	public function load_success_messages( ){
		include( $this->success_messages_file );
	}
	
	public function load_payments( ){
		include( $this->payments_file );
	}
	
	public function load_manual_bill( ){
		include( $this->manual_bill_file );
	}
	
	public function load_third_party( ){
		include( $this->third_party_file );
	}
	
	public function load_third_party_combo( ){
		$third_party_gateways = apply_filters( 'wp_easycart_admin_third_party_gateways', $this->third_party_gateways );
		foreach( $third_party_gateways as $gateway => $gateway_name ){ 
			echo '<option value="' . $gateway . '" ';
			if( get_option( 'ec_option_payment_third_party' ) == $gateway ){ 
				echo ' selected'; 
			}
			echo '>' . $gateway_name . '</option>';
		}
	}
	
	public function load_third_party_settings( ){
		$third_party_gateways = apply_filters( 'wp_easycart_admin_third_party_gateways', $this->third_party_gateways );
		foreach( $this->third_party_gateways as $gateway => $gateway_name ){
    		$this->load_third_party_payment_form( $gateway );
    	}
	}
	
	public function load_live_gateway( ){
		include( $this->live_gateway_file );
	}
	
	public function load_live_gateway_combo( ){
		$live_gateways = apply_filters( 'wp_easycart_admin_live_gateways', $this->live_gateways );
		foreach( $this->live_gateways as $gateway => $gateway_name ){ 
			echo '<option value="' . $gateway . '" ';
			if( get_option( 'ec_option_payment_process_method' ) == $gateway ){ 
				echo ' selected'; 
			}
			echo '>' . $gateway_name . '</option>';
		}
	}
	
	public function load_live_gateway_settings( ){
		$live_gateways = apply_filters( 'wp_easycart_admin_live_gateways', $this->live_gateways );
		foreach( $live_gateways as $gateway => $gateway_name ){
			$this->load_live_payment_form( $gateway );
		}
		
	}
	
	public function load_accepted_cards( ){
		include( $this->accepted_cards_file );
	}
	
	public function load_third_party_payment_form( $payment_type ){
		$file = apply_filters( 'wp_easycart_admin_payment_file', $this->payments_dir . $payment_type . '.php', $payment_type );
		if( file_exists( $file ) ){
			include( $file );
		}else{
			echo '<div class="ec_admin_settings_input ec_admin_settings_third_party_section ec_admin_settings_';
			if( get_option( 'ec_option_payment_third_party' ) == $payment_type ){
				echo 'show';
			}else{
				echo 'hide';
			}
			echo '" id="' . $payment_type . '">';
			$upgrade_icon = "dashicons-lock";
			$upgrade_title = "Enable " . $this->third_party_gateways[$payment_type];
			$upgrade_subtitle = "";
			$upgrade_checkbox_label = "Select Box to Enable " . $this->third_party_gateways[$payment_type];
			$upgrade_button_label = "Save Setup";
			include( $this->upgrade_file );
			echo '</div>';
		}
	}
	
	public function load_live_payment_form( $payment_type ){
		$file = apply_filters( 'wp_easycart_admin_payment_file', $this->payments_dir . $payment_type . '.php', $payment_type );
		if( file_exists( $file ) ){
			include( $file );
		}else{
			echo '<div class="ec_admin_settings_input ec_admin_settings_live_payment_section ec_admin_settings_';
			if( get_option( 'ec_option_payment_process_method' ) == $payment_type ){
				echo 'show';
			}else{
				echo 'hide';
			}
			echo '" id="' . $payment_type . '">';
			$upgrade_icon = "dashicons-lock";
			$upgrade_title = "Enable " . $this->live_gateways[$payment_type];
			$upgrade_subtitle = "";
			$upgrade_checkbox_label = "Select Box to Enable " . $this->live_gateways[$payment_type];
			$upgrade_button_label = "Save Setup";
			include( $this->upgrade_file );
			echo '</div>';
		}
	}
	
	public function update_manual_billing_settings( ){
		update_option( 'ec_option_use_direct_deposit', $_POST['ec_option_use_direct_deposit'] );
		update_option( 'ec_option_direct_deposit_message', stripslashes_deep( $_POST['ec_option_direct_deposit_message'] ) );
		$language = new ec_language( );
		$language->update_language_data( );
		do_action( 'wpeasycart_manual_billing_updated', esc_attr( $_POST['ec_option_use_direct_deposit'] ) );
	}
	
	public function update_third_party_selection( ){
		update_option( 'ec_option_payment_third_party', $_POST['ec_option_payment_third_party'] );
		do_action( 'wpeasycart_third_party_payment_updated', esc_attr( $_POST['ec_option_payment_third_party'] ) );
	}
	
	public function update_paypal( ){
		update_option( 'ec_option_paypal_email', stripslashes_deep( $_POST['ec_option_paypal_email'] ) );
		update_option( 'ec_option_paypal_currency_code', $_POST['ec_option_paypal_currency_code'] );
		update_option( 'ec_option_paypal_use_selected_currency', $_POST['ec_option_paypal_use_selected_currency'] );
		update_option( 'ec_option_paypal_lc', $_POST['ec_option_paypal_lc'] );
		update_option( 'ec_option_paypal_charset', $_POST['ec_option_paypal_charset'] );
		update_option( 'ec_option_paypal_weight_unit', $_POST['ec_option_paypal_weight_unit'] );
		update_option( 'ec_option_paypal_use_sandbox', $_POST['ec_option_paypal_use_sandbox'] );
		update_option( 'ec_option_paypal_collect_shipping', $_POST['ec_option_paypal_collect_shipping'] );
	}
	
	public function update_accepted_cards( ){
		$visa = $delta = $electron = $discover = $mastercard = $mastercard_debit = $american_express = $jcb = $diners = $laser = $maestro = 0; 
	
		if( isset( $_POST['ec_option_use_visa'] ) && $_POST['ec_option_use_visa'] == 1 )
			$visa = 1;
		if( isset( $_POST['ec_option_use_delta'] ) && $_POST['ec_option_use_delta'] == 1 )
			$delta = 1;
		if( isset( $_POST['ec_option_use_uke'] ) && $_POST['ec_option_use_uke'] == 1 )
			$electron = 1;
		if( isset( $_POST['ec_option_use_discover'] ) && $_POST['ec_option_use_discover'] == 1 )
			$discover = 1;
		if( isset( $_POST['ec_option_use_mastercard'] ) && $_POST['ec_option_use_mastercard'] == 1 )
			$mastercard = 1;
		if( isset( $_POST['ec_option_use_mcdebit'] ) && $_POST['ec_option_use_mcdebit'] == 1 )
			$mastercard_debit = 1;
		if( isset( $_POST['ec_option_use_amex'] ) && $_POST['ec_option_use_amex'] == 1 )
			$american_express = 1;
		if( isset( $_POST['ec_option_use_jcb'] ) && $_POST['ec_option_use_jcb'] == 1 )
			$jcb = 1;
		if( isset( $_POST['ec_option_use_diners'] ) && $_POST['ec_option_use_diners'] == 1 )
			$diners = 1;
		if( isset( $_POST['ec_option_use_laser'] ) && $_POST['ec_option_use_laser'] == 1 )
			$laser = 1;
		if( isset( $_POST['ec_option_use_maestro'] ) && $_POST['ec_option_use_maestro'] == 1 )
			$maestro = 1;
		
		update_option( 'ec_option_use_visa', $visa );
		update_option( 'ec_option_use_delta', $delta );
		update_option( 'ec_option_use_uke', $electron );
		update_option( 'ec_option_use_discover', $discover );
		update_option( 'ec_option_use_mastercard', $mastercard );
		update_option( 'ec_option_use_mcdebit', $mastercard_debit );
		update_option( 'ec_option_use_amex', $american_express );
		update_option( 'ec_option_use_jcb', $jcb );
		update_option( 'ec_option_use_diners', $diners );
		update_option( 'ec_option_use_laser', $laser );
		update_option( 'ec_option_use_maestro', $maestro );
	}
	
}
endif; // End if class_exists check

function wp_easycart_admin_payments( ){
	return wp_easycart_admin_payments::instance( );
}
wp_easycart_admin_payments( );

add_action( 'wp_ajax_ec_admin_ajax_save_third_party_selection', 'ec_admin_ajax_save_third_party_selection' );
function ec_admin_ajax_save_third_party_selection( ){
	wp_easycart_admin_payments( )->update_third_party_selection( );
	die( );
}

add_action( 'wp_ajax_ec_admin_ajax_save_direct_deposit', 'ec_admin_ajax_save_direct_deposit' );
function ec_admin_ajax_save_direct_deposit( ){
	wp_easycart_admin_payments( )->update_manual_billing_settings( );
	die( );
}

add_action( 'wp_ajax_ec_admin_ajax_save_paypal', 'ec_admin_ajax_save_paypal' );
function ec_admin_ajax_save_paypal( ){
	wp_easycart_admin_payments( )->update_third_party_selection( );
	wp_easycart_admin_payments( )->update_paypal( );
	die( );
}

add_action( 'wp_ajax_ec_admin_ajax_save_accepted_cards', 'ec_admin_ajax_save_accepted_cards' );
function ec_admin_ajax_save_accepted_cards( ){
	wp_easycart_admin_payments( )->update_accepted_cards( );
	die( );
}